from RoboidControl.Participants.ParticipantUDP import ParticipantUDP
from RoboidControl.Messages.ParticipantMsg import ParticipantMsg
from RoboidControl.Messages.NetworkIdMsg import NetworkIdMsg
from RoboidControl.Messages.ThingMsg import ThingMsg
from RoboidControl.Things.TemperatureSensor import TemperatureSensor
from RoboidControl.Thing import Thing

from typing import Optional

class SiteServer(ParticipantUDP):
    """! A site server is a participant which provides a shared simulated environment 
    """

    name = "Site Server"

#region Init

    def __init__(self, port: int = 7681):
        """! Create a new site server
        @param port The port of which to receive the messages
        """
        super().__init__(ip_address = "127.0.0.1", port = port)
        self.isolated = False   # site servers are never isolated
        self.publishInterval = 0

#endregion Init

#region Update
#endregion Update

#region Receive

    def ProcessParticipantMsg(self, sender, msg):
        ParticipantUDP.ProcessParticipantMsg(sender, msg)
        if msg.network_id != sender.network_id:
            self.Send(sender, sender.network_id)
    
    def ProcessNetworkIdMsg(self, sender, msg):
        pass

    def ProcessThingMsg(self, sender, msg: ThingMsg):
        thing: Optional[Thing] = sender.Get(msg.thing_id)
        if thing is None:
            Thing(sender, msg.thing_type, msg.thing_id)
        
        if msg.parent_id != 0:
            thing.parent = sender.Get(msg.parent_id)
            if thing.parent is not None:
                print(f'Could not find parent [{msg.network_id}/{msg.parent_id}]')
        else:
            thing.parent = None

#endregion Receive    